#!/usr/bin/python3.5
#
# This is the Python module implementing access to the DHT11
# U. Raich UCC, Nov.2017
import sys
import RPi.GPIO as GPIO
import time

#
# dht11Lib error codes
#
SUCCESS=                0
INIT_FAILED=           -1
MISSING_INIT=          -2
NO_MEAS_YET=           -3 
NO_VALID_MEAS_YET=     -4
MEM_ALLOC_ERROR=       -5
PROTOCOL_ERROR=        -6
BAD_CHECKSUM=          -7
BAD_TIME_STAMP=       -10

DHT11_PIN=             17

PROTOCOL_TIMEOUT=     100
PROTOCOL_SIZE=        800
PROTOCOL_SIZE_MARGIN= 200

MAX_UNCHANGED_COUNT=  100
HIGH=                   1
LOW=                    0
ON=                     1
OFF=                    0

errorMsg={ SUCCESS:           "library call returned successfully",
           INIT_FAILED:       "Failed to initialize the pigpiod connection",
           MISSING_INIT :     "init was not called yet",
           NO_MEAS_YET:       "No measurement was made yet",
           NO_VALID_MEAS_YET:  "No valid measurement was made yet",
           MEM_ALLOC_ERROR:   "Could not allocate enough memory to hold the data",
           PROTOCOL_ERROR:    "Protocol error: the data stream was probably not read entirely",
           BAD_CHECKSUM:      "Checksum error on protocol data",
           BAD_TIME_STAMP:    "The time stamp was invalid",
           }


def printError(err):
    try:
        print(errorMsg[err])
    except KeyError:
        print("Unknown error code")

# initialize the rpi.GPIO library

def open():
    GPIO.setmode(GPIO.BCM)
    GPIO.setwarnings(True)
    return SUCCESS

def close():
    GPIO.cleanup()   
def measurement():
    global debug
    global dht11Data

    dht11Data = []

#
# set the dht11 control pin to output to start the measurement
#
    GPIO.setup(DHT11_PIN,GPIO.OUT)  
    GPIO.output(DHT11_PIN,LOW)
    time.sleep(0.018 )          # wait for 18 ms
    GPIO.output(DHT11_PIN,HIGH)
#    time.sleep(0.000040)        # wait for 40 us

# now switch the pin to input
    GPIO.setup(DHT11_PIN,GPIO.IN)
    
    last = -1
    count = 0
    while True:
        current=GPIO.input(DHT11_PIN)
        dht11Data.append(current)
        if (current != last):
            last = current
            count = 0
        else:
            count += 1
            if count > MAX_UNCHANGED_COUNT:
                break

##    for i in range(0,len(dht11Data)):
##        print(dht11Data[i])
    return SUCCESS

#
# this decodes the protocol read from the dht11
# it first waits for the response signal from the DHT11 to pass
# and then reads the protocol data bit by bit
#
def decode():

# this function reads a single bit from the protocol
# pointer is an index into the data array which is pass back to the calling
# decode() function (use a nonlocal variable)
#
    def __readBit():
        global dht11Data
        nonlocal pointer

        if int(dht11Data[pointer+10]) == HIGH:
            dataBit=1
        else:
            dataBit=0
            
        for index in range(0,50):
#            print("dht11Data[",pointer, "] : ",dht11Data[pointer]);
            if int(dht11Data[pointer]) == HIGH:            
                pointer += 1
                if pointer > len(dht11Data):
                    return PROTOCOL_ERROR
            else:
                break
        pointer += 1  
        if index == 50:
            print("protocol error")
            return PROTOCOL_ERROR
            
        for index in range(0,50):
 #           print("dht11Data[",pointer, "] : ",dht11Data[pointer]);
            if int(dht11Data[pointer]) == LOW:             
                pointer += 1
                if pointer > len(dht11Data):
                    return PROTOCOL_ERROR               
            else:
                break
            
        if index == 50:
            print("protocol error")           
            return PROTOCOL_ERROR   
        else:
            print("bit: ",dataBit)
            return dataBit
#        
# end of readBit         
#       

# must advance this far enough to skip to start with the response signal
# from the DHT11

    pointer=30
#
# wait until response from dht11 is finished
#
    global dht11Data
    for index in range(0,50):
        if int(dht11Data[pointer]) == HIGH:
            pointer += 1
            continue
#    print("# pointer when signal goes down: ",pointer)
    if index == 50:
        print("Did not see DHT11 start signal")
        return PROTOCOL_ERROR
    pointer += 1
    for index in range(0,50):
        if int(dht11Data[pointer]) == LOW:
            pointer += 1
            continue
#    print("# pointer when signal goes high again: ",pointer)
    if index == 50:
        print("Did not see DHT11 start signal")
        return PROTOCOL_ERROR        
    pointer +=1
#    print("# pointer after DHT11 start signal: ",pointer)

    humTempData = []
    for humTemp in range(0,5):
        data = 0
        for bits in range(0,8):
#            print("pointer: ",pointer)
            data <<= 1
            data |= int(__readBit())
        print("data byte: ",format(data,'02x'))
        humTempData.append(data);
    for index in range(0,len(humTempData)):
        print("data[",index,"] = 0x",format(humTempData[index],'02x'))
    checksum = 0
    for index in range(0,4):
        checksum += humTempData[index]
    checksum &= 0xff
    print("Checksum calculated: ",format(checksum,'02x'),
           " transmitted: ",format(humTempData[4],'02x'))
    print("Humidity: ",humTempData[0], " Temperature: ",humTempData[2])
    
    
